#include "gp32.h"
#include "fileio.h"
#include "zda.h"
#include <stdlib.h>

#include "filezda.h"

#define BASEDIR "dev0:\\GPMM\\ADIC2K4"

u8 *zda_full_header = NULL;
GPFILE *zda_file = NULL;

int openZDA(const char *path)
{
	u8 header_12bytes[12];
	int real_headersize;
	char name[64];

	sprintf(name,"%s\\%s",BASEDIR,path);
	zda_file = smc_fopen(name,"r");
	if (zda_file==NULL)
		return 1;
	smc_fread(header_12bytes,12,1,zda_file);
	real_headersize=gp_zda_headersize(header_12bytes);

	zda_full_header = (char*)malloc(real_headersize);
	smc_fseek(zda_file,0,SEEK_SET);
	smc_fread(zda_full_header,1,real_headersize,zda_file);
	return 0;
}

void closeZDA()
{
	smc_fclose(zda_file);
	free(zda_full_header);
	zda_file = NULL;
	zda_full_header = NULL;
}

u8 *uncompressZDA(const char* filename,int* usize)
{
	u8 *compressed;
	u8 *uncompressed;

	int csize  = gp_zda_csize (zda_full_header, filename);
	int offset = gp_zda_offset(zda_full_header, filename);

	*usize = gp_zda_usize (zda_full_header, filename);

	compressed   = (char*)malloc(csize);
	uncompressed = (char*)malloc(*usize);

	smc_fseek(zda_file,offset,SEEK_SET);
	smc_fread(compressed,csize,1,zda_file);

	if (gp_uncompressZDA(uncompressed,usize,compressed,csize)==1)
	{
		free(compressed);
		free(uncompressed);
		return NULL;
	}
	free(compressed);
	return uncompressed;
}

u8* loadZDAFile(const char *filename,const char *ext,int *size)
{
	int s,*sz;
	char name[16];
	sz = size==NULL?&s:size;
	sprintf(name,"%s.%s",filename,ext);
	return uncompressZDA((const char*)name,sz);
}
