#include "gp32.h"
#include <stdlib.h>
#include <string.h>

// Include from Yeti 3d engine
#include "yeti.h"
#include "viewports.h"
#include "game.h"

#include "gfx.h"
#include "filezda.h"

// Declare Yeti 3D structure engine and viewport
framebuffer_t framebuffer;
yeti_t yeti;

YETI_ROM texture_t *textures;
palette_t *palette;
YETI_ROM lua_t *lua;
int nbtex;

#define NB_OBJECTS 3
const u8 *obj_name[MAX_OBJECTS] = {"knight","blade","cobra"};
object_t o3d_objects[MAX_OBJECTS];
const u8 *obj_wname[MAX_OBJECTS] = {"fireblad",NULL,NULL};
object_t o3d_weapons[MAX_OBJECTS];

// GP32 port info !!!
#define YETI_GP32_PORT_STR_TITLE      "Yeti3D Engine -> GP32 PORT"
#define YETI_GP32_PORT_VERSION        "0.0.3.7"
#define YETI_GP32_PORT_STR_AUTOR      "GAZIN Matthieu"
#define YETI_GP32_PORT_STR_CAPTION    (YETI_GP32_PORT_STR_TITLE " " YETI_GP32_PORT_STR_VERSION " - " YETI_GP32_PORT_STR_AUTOR)

int key_pressed;	// For input check
int byebye = 0;		// on quitte ;)

#define SHOW_FPS

#ifdef SHOW_FPS
// some fps count
unsigned long fps = 0;
unsigned long fps_tick;
char fps_str[100];
#endif

// main boucle
void gp32_to_yeti3D(void);

// loader fichier extern
int load_texture_and_pal_files(char *name,int nbobj);
void gp32_load_object(yeti_t* yeti,const char* filename,object_t*objects);
void compute_lua();

void yetimain()
{    
	char tmp_buffer[255];
	int i;
	rom_map_t *rommap;

	clearScreen();
	console_write("---------------------------------------",COLOR_WHITE);
	console_write(YETI_GP32_PORT_STR_TITLE,COLOR_YELLOW);
	sprintf(tmp_buffer,"Version : %s  by %s",YETI_GP32_PORT_VERSION,YETI_GP32_PORT_STR_AUTOR);
	console_write(tmp_buffer,COLOR_YELLOW);
	console_write("---------------------------------------",COLOR_WHITE);

	// Bon ca peut parraitre con vue ke on est deja la mais bon c pour le fun on va dire ke c'est bon quand meme :p
	console_write("Init buffers OK",COLOR_GREEN);
	console_write("Enable CPU cache OK",COLOR_GREEN);
	console_write("CPU overclock OK",COLOR_GREEN);

	console_write("Loading Data ...",COLOR_GREEN); 

	if (openZDA("DATA1.ZDA"))
		error("BAD DATA FILE !!!");

	// chargement des textures
	nbtex = load_texture_and_pal_files("test",4);

	console_write_no_lineup("Loading objects",COLOR_ORANGE);
	for (i=0;i<NB_OBJECTS;i++)
		gp32_load_object(&yeti,obj_name[i],&o3d_objects[i]);
	for (i=0;i<NB_OBJECTS;i++)
	{
		if (obj_wname[i])
			gp32_load_object(&yeti,obj_wname[i],&o3d_weapons[i]);
	}
	console_write("Loading objects OK !!!",COLOR_GREEN);

	compute_lua();

	console_write_no_lineup("Yeti3D Engine init",COLOR_ORANGE);
	// init engine
	yeti_init(&yeti, &framebuffer, &framebuffer, textures, palette, lua);
	console_write("Yeti3D Engine init OK !!!",COLOR_GREEN);

	console_write_no_lineup("Loading Map",COLOR_ORANGE);
	rommap = (rom_map_t*)loadZDAFile("test","y3d",NULL);
	yeti_load_map(&yeti, rommap);
	free(rommap);
	console_write("Loading Map OK !!!",COLOR_GREEN);

	closeZDA();

	console_write_no_lineup("Game Engine init",COLOR_ORANGE);
	game_init(&yeti);

	console_write("Game Engine init !!!",COLOR_GREEN);

	// Memory left after all ?
/*
	sprintf(tmp_buffer,"Memory available : %d kb",gm_availablesize()/1024);
	console_write(tmp_buffer,COLOR_GREEN);
*/
	console_write("Running ....",COLOR_GREEN);

#ifdef SHOW_FPS
	fps_str[0] = 0;
#endif

i=50;
	while(!byebye)
	{
		// Loop of the gp32 port
if (--i==0)
{
if(yeti.cells[20][39].bot<yeti.cells[20][39].top)
yeti.cells[20][39].bot+=16;
i=50;
}
		gp32_to_yeti3D();   
	}

	for (i=0;i<MAX_OBJECTS;i++)
	{
		free(o3d_objects[i].radius);
		free(o3d_objects[i].height);
		free(o3d_objects[i].data);
	}
	free(textures);
	free(palette);
	free(lua);
}

int getTickCount()
{
	return (gp_getRTC()*1000)>>6;
}

void gp32_to_yeti3D(void)
{
	static unsigned long Time = 0;

	// for screen copie
	u32 tick = getTickCount();

	if ((int)(Time - tick) < 0)
	{
		Time = tick + YETI_VIEWPORT_INTERVAL;

		yeti.keyboard.up=yeti.keyboard.down=yeti.keyboard.left=yeti.keyboard.right=yeti.keyboard.a=yeti.keyboard.b=yeti.keyboard.l=yeti.keyboard.r=0;

		key_pressed=gp_getButton(); // Get input data

		// bye bye !!!
		if (key_pressed & BUTTON_START)
			byebye = 1;

		if (key_pressed & BUTTON_UP)
			yeti.keyboard.up     = 1;
		if (key_pressed & BUTTON_DOWN)
			yeti.keyboard.down   = 1;
		if (key_pressed & BUTTON_LEFT)
			yeti.keyboard.left   = 1;
		if (key_pressed & BUTTON_RIGHT)
			yeti.keyboard.right  = 1;
		if (key_pressed & BUTTON_A)
			yeti.keyboard.a  = 1;
		if (key_pressed & BUTTON_B)
			yeti.keyboard.b  = 1;
		if (key_pressed & BUTTON_L)
			yeti.keyboard.l  = 1;
		if (key_pressed & BUTTON_R)
			yeti.keyboard.r  = 1;

		// Engine loop
		framebuffer.pixels = (rgb555_t*)getCurrentBuffer();
		game_loop(&yeti);

#ifdef SHOW_FPS
		if (tick-fps>1000)
		{
			sprintf(fps_str,"fps=%ld",fps_tick);
			fps_tick = 0;
			fps = tick;
		}
		else
			fps_tick++;
		drawTextAt(fps_str,5,5,COLOR_WHITE);
#endif
		flip(0);
	}
}

int	lua_rgb (int r, int g, int b)
{
	int	r1 = (MIN (r, 255) * 31) / 255;
	int	g1 = (MIN (g, 255) * 31) / 255;
	int	b1 = (MIN (b, 255) * 31) / 255;
	return ( ( ( r1 ) << 11 )  |  ( ( g1 ) << 6 )  |  ( ( b1 ) << 1 ) | 1);
}

// only indicate general file name extension raw and pal will be set
int load_texture_and_pal_files(char *name,int nbobj)
{
	u8* data;
	int num_tex,fsize=0;
	char buf[64];

	console_write_no_lineup("Loading Textures",COLOR_ORANGE);
	data = loadZDAFile(name,"RAW",&fsize);
	num_tex=fsize/(TXSIZE*TXSIZE);
	textures = (texture_t*)malloc((num_tex+nbobj)*TXSIZE*TXSIZE);
	memcpy(textures,data,num_tex*TXSIZE*TXSIZE);
	free(data);
	sprintf(buf,"Loading Textures (%i founds) OK !!!",(int)num_tex);
	console_write(buf,COLOR_GREEN);

	console_write_no_lineup("Loading pallette",COLOR_ORANGE);
	data = loadZDAFile(name,"PAL",NULL);
	palette = (palette_t*)malloc((num_tex+nbobj)*768);
	memcpy(palette,data,num_tex*768);
	free(data);
	lua = (lua_t*)malloc((num_tex+nbobj)*sizeof(lua_t));
	console_write("Loading pallette OK !!!",COLOR_GREEN);

	return num_tex;
}

void compute_lua()
{
	int a,i,j;
	for(j=0;j<nbtex;j++)
	{
		for(a=0;a<64;a++)
		{
			for(i=0;i<256;i++) 
			{
				lua[j][a][i]=lua_rgb((palette[j][i][0]*a)>>4,(palette[j][i][1]*a)>>4,(palette[j][i][2]*a)>>4); 
			}
		}
	}
}

void gp32_load_object(yeti_t* yeti,const char* filename,object_t* obj)
{
	u8 *tex,*ptr;
	int nbf,nba,psize,size;

	obj->data = loadZDAFile(filename,"o3d",&size);
	psize = (*obj->data)*3;
	tex = obj->data+psize+1;
	obj->anims = tex+TXSIZE*TXSIZE;
	nba = *obj->anims;
	nbf = obj->anims[nba+1]*2;
	obj->radius = (u16*)malloc(nbf);
	ptr = obj->anims+nba+2;
	memcpy(obj->radius,ptr,nbf);
	ptr += nbf;
	obj->height = (u16*)malloc(nbf);
	memcpy(obj->height,ptr,nbf);
	obj->model = ptr+nbf;
	*(obj->model+3) = nbtex;
	memcpy(textures[nbtex],tex,TXSIZE*TXSIZE);
	memcpy(palette[nbtex++],obj->data+1,psize);
}
